/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.sf.jftp;

import net.sf.jftp.gui.framework.*;
import net.sf.jftp.gui.*;
import net.sf.jftp.config.Settings;
import net.sf.jftp.util.*;
import net.sf.jftp.net.*;

import java.net.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class JFtp extends JPanel implements WindowListener, ComponentListener, Logger
{
    public static boolean mainUsed = false;
    public static StatusPanel statusP;
    public static JLabel statusL = new JLabel("Welcome to JFtp...                                                            ");
    public static JFrame mainFrame;
    public static DirPanel localDir;
    public static DirPanel remoteDir;
    public static DownloadList dList = new DownloadList();
    public static boolean uiBlocked = false;
    
    private boolean initSize = true;
    private String oldText = "";

    /** JSplitPane that holds the local and remote DirPanels */
    private JSplitPane dirP = null;

    /** JSplitPane that holds the directory panes and the log/dl JSplitPane */
    private JSplitPane workP = null;
    
    /** JSplitPane that holds the log download parts */
    private JSplitPane logP = null;

    private static JScrollPane logSp;
    public static JTextArea log;
    
    public HostChooser hc;

    public JFtp()
    {
        Log.setLogger(this);
        // we have jesktop-environment
        statusP.remove(statusP.close);
        init();
    }

    public JFtp(boolean mainUsed)
    {
        Log.setLogger(this);
        this.mainUsed = mainUsed;
        init();
    }

    public void init()
    {
        setLayout(new BorderLayout());

        setBackground(GUIDefaults.mainBack);
        setForeground(GUIDefaults.front);

        statusP = new StatusPanel(this);
        add("North",statusP);
        localDir = new DirPanel(Settings.defaultWorkDir,"local");
        remoteDir = new DirPanel("/","remote");
	remoteDir.dList = dList;
	ConnectionHandler.setConnectionListener(remoteDir);

        Dimension d = Settings.getWindowSize();
        setPreferredSize(d);
        setSize(d);
        int width = (int)d.getWidth();
        int height = (int)d.getHeight();
        dirP= new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, localDir, remoteDir);

	log = new JTextArea();
        log.setBackground(GUIDefaults.back);
        log.setForeground(GUIDefaults.front);
        log.setEditable(false);
        logSp = new JScrollPane(log);
	
        logP = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, logSp, dList);

        workP = new JSplitPane(JSplitPane.VERTICAL_SPLIT, dirP, logP);
        workP.setSize(new Dimension((int)width, (int)height));
        add("Center", workP);

        dirP.setMinimumSize(new Dimension((int)width, (int)(height * 0.60)));
        dirP.setPreferredSize(new Dimension((int)width, (int)(height * 0.75)));
        dirP.setSize(new Dimension((int)width, (int)(height * 0.75)));
        dirP.setDividerLocation(0.5);
        logSp.setMinimumSize(new Dimension((int)(width/1.6666), (int)(height * 0.20)));
        logSp.setPreferredSize(new Dimension((int)(width/1.6666), (int)(height * 0.25)));
        logSp.setSize(new Dimension((int)(width/1.6666), (int)(height * 0.25)));
        dList.setMinimumSize(new Dimension((int)(width/2.5), (int)(height * 0.20)));
        dList.setPreferredSize(new Dimension((int)(width/2.5), (int)(height * 0.25)));
        dList.setSize(new Dimension((int)(width/2.5), (int)(height * 0.25)));
        workP.setDividerLocation(0.6);

        addComponentListener(this);
        componentResized(new ComponentEvent(dirP,0));
        componentResized(new ComponentEvent(logSp,0));
        componentResized(new ComponentEvent(workP,0));
//debugComponent(localDir);
//debugComponent(remoteDir);
//debugComponent(dirP);
//debugComponent(workP);
//debugComponent(logSp);
        validate();
        setVisible(true);

        if(!mainUsed)
        {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    chooseHost();
                }
            });
        }
        else
        {
            chooseHost();
        }


    }

    protected void chooseHost() {
        hc = new HostChooser(this);
        if(!mainUsed)
        {
	    hc.update();
        }
    }

    public static String getHost()
    {
        return statusP.getHost();
    }

    public static void setHost(String which)
    {
        statusP.setHost(which);
    }

    public static void localUpdate()
    {
        localDir.setPath(localDir.path);
    }

    public static void remoteUpdate()
    {
    	FtpConnection con = ConnectionHandler.getConnection();
        con.chdir(con.getPWD());
    }

    public void windowClosing(WindowEvent e)
    {
        Settings.setProperty("jftp.window.width", this.getWidth());
        Settings.setProperty("jftp.window.height", this.getHeight());
        Settings.setProperty("jftp.window.x", (int)this.getLocationOnScreen().getX());
        Settings.setProperty("jftp.window.y", (int)this.getLocationOnScreen().getY());
        Settings.save();

	if(remoteDir.con != null && remoteDir.con.isConnected())
	{
        	try {
            		ConnectionHandler.getConnection().disconnect();
        	} catch(Exception ex) {
        	}
	}

        System.exit(0);
    }

    public void windowClosed(WindowEvent e)
    {}

    public void windowActivated(WindowEvent e)
    {}

    public void windowDeactivated(WindowEvent e)
    {}

    public void windowIconified(WindowEvent e)
    {}

    public void windowDeiconified(WindowEvent e)
    {}

    public void windowOpened(WindowEvent e)
    {}

    public void componentHidden(ComponentEvent e)
    {}

    public void componentMoved(ComponentEvent e)
    {}

    public void componentShown(ComponentEvent e)
    {}

    public void componentResized(ComponentEvent e)
    {
        localDir.gui(true);
        remoteDir.gui(true);
        validate();
    }

    public static void main(String argv[])
    {
    	Settings.enableResuming = true;
        JFtp jftp = new JFtp(true);

        mainFrame = new JFrame();
		Point p = Settings.getWindowLocation();
        mainFrame.setLocation(p);
        mainFrame.setTitle(Settings.title);
        mainFrame.setResizable(Settings.resize);
        mainFrame.addWindowListener(jftp);
        Image icon = HImage.getImage(jftp, Settings.iconImage);
        mainFrame.setIconImage(icon);
        mainFrame.setFont(GUIDefaults.font);
        mainFrame.getContentPane().setLayout(new BorderLayout());
        mainFrame.getContentPane().add("Center", jftp);
        mainFrame.pack();
        mainFrame.validate();

	if(argv.length > 0)
	{
		jftp.hc.update(argv[0]);
	}
	else
	{
		jftp.hc.update();
	}
    }

    private void log(String msg)
    {
        if(msg.startsWith("200") || msg.startsWith("227"))
        {
            if(msg.indexOf("NOOP") > 0 || msg.indexOf("Type") > 0 ||
                    msg.indexOf("MODE") > 0 || msg.indexOf("Passive") > 0)
            {
                if(Settings.hideStatus)
                {
                    return;
                }
            }
        }

        log.append(" "+msg+"\n");
        //System.out.println(msg);

        JScrollBar bar = logSp.getVerticalScrollBar();
        bar.setValue(bar.getMaximum());
        logSp.invalidate();
        logSp.validate();

        SwingUtilities.invokeLater(new Runnable() {
                public void run() {
		    invalidate();
		    validate();
		    JFtp.localDir.gui(true);
		    //JFtp.remoteDir.invalidate();
		    //JFtp.remoteDir.validate();
		    JFtp.remoteDir.gui(true);
                }
        });
    }

    private void log(String msg, Throwable throwable)
    {
        PrintWriter p = new PrintWriter(new StringWriter());
        throwable.printStackTrace( p );
        log(msg);
        log(p.toString());
    }

    public void debug(String msg)
    {
       log(msg);
    }

    public void debug(String msg, Throwable throwable)
    {
        log(msg, throwable);
    }

    public void warn(String msg)
    {
        log(msg);
    }

    public void warn(String msg, Throwable throwable)
    {
        log(msg, throwable);
    }

    public void error(String msg)
    {
        log( msg );
    }

    public void error(String msg, Throwable throwable)
    {
        log( msg, throwable );
    }

    public void info(String msg)
    {
        log( msg );
    }

    public void info(String msg, Throwable throwable)
    {
        log( msg, throwable );
    }

    public void fatal(String msg)
    {
        log( msg );
    }

    public void fatal(String msg, Throwable throwable)
    {
        log( msg, throwable );
    }

    public void debugSize(int size, boolean recv, boolean last, String file)
    {
	if(initSize)
	{
		initSize = false;
		oldText = log.getText();
	}

	int total = 0;
	ListModel l =  null;

	if(recv) l = remoteDir.jl.getModel();
	else l = localDir.jl.getModel();

	for(int i=0; i<l.getSize(); i++)
	{
	 DirEntry d = (DirEntry) l.getElementAt(i);

	 file = file.substring(file.lastIndexOf(File.separatorChar)+1);
	 //System.out.println(d + ":" + file);

	 if(d.toString().trim().equals(file.trim()))
	 {
	 	total = d.getRawSize();
	 	break;
	 }
	}

	int s = size/1024;
	int t = total/1024;

	if(s > 0)
	{
	 if(recv) log.setText(file + ": received " + s + " kb of " + t + " kb so far.");
	 else log.setText(file + ": sent " + s + " kb of " + t + " kb so far.");

	 if(last)
	 {
	 	log.setText(oldText);
	 	debug(file + ": transfer complete. Exact size was " + (size+1) + " bytes.");
		initSize = true;
	 }
	}
    }

    /*
    public Insets getInsets()
    {
        Insets in = super.getInsets();
        return new Insets(in.top+2,in.left+2,in.bottom+2,in.right+2);
    }
    */
}
