/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.sf.jftp.gui;

import net.sf.jftp.gui.framework.*;
import net.sf.jftp.config.*;
import net.sf.jftp.net.*;
import net.sf.jftp.util.*;
import net.sf.jftp.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.StringTokenizer;

public class DirPanel extends TransferComponent implements ActionListener, ListSelectionListener, ConnectionListener
{
    private JButton deleteButton;
    private JButton mkdirButton;
    private JButton cmdButton;
    private JButton refreshButton;
    private JButton cdButton;
    private JButton uploadButton;
    private JButton downloadButton;

    public int length = 0;
    public String[] files;

    private DirCanvas label = new DirCanvas();
    private boolean pathChanged = true;
    private boolean firstGui = true;
    private int pos=0;
    private JPanel p = new JPanel();
    private JPanel buttonPanel = new JPanel();
    private JPanel currDirPanel = new JPanel();
    public JList jl = new JList();
    private DefaultListModel jlm;
    private JScrollPane jsp = new JScrollPane(jl);
    private int tmpindex = -1;
    public DownloadList dList = null;
    public FtpConnection con = null; // only for connectionfailed/JFtp

    private static final String deleteString = "rm";
    private static final String mkdirString = "mkdir";
    private static final String refreshString = "fresh";
    private static final String cdString = "cd";
    private static final String cmdString = "cmd";
    private static final String downloadString = "<-";
    private static final String uploadString = "->";

    public DirPanel(String path, String type)
    {
        this.path = path;
        this.type = type;
    }

    public void gui_init()
    {
	setLayout(new BorderLayout());

        currDirPanel = new JPanel();

        label.setText(type + ": " + cutPath(path));
        label.setSize(getSize().width-10, 24);
        currDirPanel.add(label);
        currDirPanel.setSize(getSize().width-10, 32);
        label.setSize(getSize().width-20, 24);

	p.setLayout(new BorderLayout());
	p.add("North", currDirPanel);

        ImageIcon icon = HImage.getImageIcon(Settings.downloadImage,downloadString);
        downloadButton = new JButton(icon);
        downloadButton.setActionCommand(downloadString);
        downloadButton.addActionListener(this);
        downloadButton.setToolTipText("Download selected");

        icon = HImage.getImageIcon(Settings.deleteImage,deleteString);
        deleteButton = new JButton(icon);
        deleteButton.setActionCommand(deleteString);
        deleteButton.addActionListener(this);
        deleteButton.setToolTipText("Delete selected");
        //
        icon = HImage.getImageIcon(Settings.mkdirImage,mkdirString);
        mkdirButton = new JButton(icon);
        mkdirButton.setActionCommand(mkdirString);
        mkdirButton.addActionListener(this);
        mkdirButton.setToolTipText("Create directory");
        //
        icon = HImage.getImageIcon(Settings.refreshImage,refreshString);
        refreshButton = new JButton(icon);
        refreshButton.setActionCommand(refreshString);
        refreshButton.addActionListener(this);
        refreshButton.setToolTipText("Refresh");
        //
        icon = HImage.getImageIcon(Settings.cdImage,cdString);
        cdButton = new JButton(icon);
        cdButton.setActionCommand(cdString);
        cdButton.addActionListener(this);
        cdButton.setToolTipText("Change directory");
        //
        icon = HImage.getImageIcon(Settings.uploadImage,uploadString);
        uploadButton = new JButton(icon);
        uploadButton.setActionCommand(uploadString);
        uploadButton.addActionListener(this);
        uploadButton.setToolTipText("Upload selected files");

        //
        buttonPanel = new JPanel();
        if (type.equals("remote"))
        {
            buttonPanel.add(downloadButton);
        }
        buttonPanel.add(deleteButton);
        buttonPanel.add(mkdirButton);

        if(type.equals("remote"))
        {
            icon = HImage.getImageIcon(Settings.cmdImage,cmdString);
            cmdButton = new JButton(icon);
            cmdButton.setActionCommand(cmdString);
            cmdButton.addActionListener(this);
	    cmdButton.setToolTipText("Execute remote command");
            buttonPanel.add(cmdButton);
        }

        buttonPanel.add(refreshButton);
        buttonPanel.add(cdButton);
        if (type.equals("local"))
        {
            buttonPanel.add(uploadButton);
        }
        buttonPanel.setVisible(true);

        buttonPanel.setSize(getSize().width-10, 32);
	p.add("South", buttonPanel);
	add("North", p);

        setDirList(true);
        jlm=new DefaultListModel();
        jl=new JList(jlm);
        jl.setCellRenderer(new DirCellRenderer());
        jl.setVisibleRowCount(Settings.visibleFileRows);
        jl.addListSelectionListener(this);

        // add this becaus we need to fetch only doubleclicks
        MouseListener mouseListener = new MouseAdapter()
                                      {
                                          public void mouseClicked(MouseEvent e)
                                          {
					   if(JFtp.uiBlocked) return;

						//System.out.println("DirEntryListener::");
                                              if (e.getClickCount() == 2)
                                              {
                                                  if(type.equals("local"))
                                                  {
                                                      //System.out.println("2xList selection: "+jl.getSelectedValue().toString());
                                                      int index = jl.getSelectedIndex()-1;

						      // mousewheel bugfix, ui refresh bugfix
						      if(index < -1 || dirEntry == null || dirEntry.length < index || dirEntry[index] == null) return;

                                                      String tgt=(String)jl.getSelectedValue().toString();
                                                      //System.out.println("List selection: "+index);
                                                      if (index < 0)
                                                      {
                                                          setPath(path+tgt);
                                                      }
                                                      else if (dirEntry[index].isDirectory())
                                                      {
                                                          setPath(path+tgt);
                                                      }
                                                      else
                                                      {
                                                          blockedTransfer(index);
                                                      }
                                                  }
                                                  else
                                                  {
                                                      //System.out.println("2xList selection: "+jl.getSelectedValue().toString());
                                                      int index = jl.getSelectedIndex()-1;

						      // mousewheel bugfix
						      if(index < -1) return;

						      String tgt=(String)jl.getSelectedValue().toString();

                                                      if (index < 0)
                                                      {
                                                          ConnectionHandler.getConnection().chdir(path+tgt);
                                                      }
                                                      else if (dirEntry[index].isDirectory())
                                                      {
                                                          ConnectionHandler.getConnection().chdir(path+tgt);
                                                      } else if(dirEntry[index].isLink())
						      {
						      	  if(!ConnectionHandler.getConnection().chdir(path+tgt))
							  {
							  	blockedTransfer(index);
							  }
						      }
                                                      else
                                                      {
                                                          blockedTransfer(index);
                                                      }
                                                  }
                                              }
                                          }
                                      };
        jl.addMouseListener(mouseListener);

        jsp = new JScrollPane(jl);
        jsp.setSize(getSize().width-20, getSize().height-72);
        add("Center", jsp);
        jsp.setVisible(true);
        setVisible(true);
    }

    public void setViewPort()
    {
    }


    public void gui(boolean fakeInit)
    {
        if (firstGui)
        {
            gui_init();
            firstGui=false;
        }
        label.setText(type + ": "+cutPath(path));
        if (!fakeInit)
        {
            setDirList(false);
        }
	invalidate();
	validate();
    }

    public void setDirList(boolean fakeInit)
    {
        jlm = new DefaultListModel();
        DirEntry dwn = new DirEntry("..",this);
        dwn.setDirectory();
        jlm.addElement(dwn);
	
        // directory init

        if(!fakeInit)
        {
            if(pathChanged)
            {
                pathChanged = false;
                DirLister dir = new DirLister(path,type);

                while(!dir.finished) LocalIO.pause(10);

                if(dir.isOk())
                {
                    length = dir.getLength();
                    dirEntry = new DirEntry[length];
                    files = dir.list();
                    String fSize[] = dir.sList();
		    String perms[] = dir.getPermissions();

		    // --------- sorting aphabetically ------------
		    if(Settings.sortDir)
	            {

		     if(type.equals("local"))
		     {
		     	LocalIO.sortStrings(files);
		     }
		     else
		     {

		      String[] tmpx = new String[length];

		      //if(fSize != null) System.out.println(":"+length+":"+fSize.length+":"+files.length);

		      int pLength = length;
		      if(perms != null) pLength = perms.length;

		      //System.out.println(files.length + ":" + fSize.length+":"+ pLength + ":"+ length);

		      if(fSize.length != files.length || pLength != files.length || length != files.length) System.out.println("Sort mismatch - hopefully ignoring it...");

                      for(int x=0; x<length; x++)
                      {
			tmpx[x] = files[x] + "@@@" + fSize[x];
		      }
		      LocalIO.sortStrings(tmpx);

                      for(int y=0; y<length; y++)
                      {
			files[y] = tmpx[y].substring(0,tmpx[y].lastIndexOf("@@@"));
			fSize[y] = tmpx[y].substring(tmpx[y].lastIndexOf("@@@")+3);
		      }
		     }
		    }
		    // ----------- end sorting --------------------
		    
                    for(int i=0; i<length; i++)
                    {
                        //System.out.println(files[i]);
                        dirEntry[i] = new DirEntry(files[i],this);
			if(files == null || files[i] == null)
			{
				System.out.println("Critical error, files or files[i] is null!\nPlease report when and how this happened...");
				System.exit(0);
			}
			
			if(perms != null && perms[i].equals("n")) dirEntry[i].setPermission(false);
			
                        jlm.addElement(dirEntry[i]);

                        if(type.equals("local"))
                        {
                            File f = new File(path+files[i]);

                            if(f.isDirectory())
                            {
			        if(dirEntry[i] == null) { System.out.println("\n\n\nd null!!!"); continue; }
				if(dirEntry[i].file == null) System.out.println("\n\n\nf null!!!");
                                dirEntry[i].setDirectory();
                                if(!dirEntry[i].file.endsWith("/"))
                                    dirEntry[i].file = dirEntry[i].file + "/";
                            }
                        }
                        if(type.equals("remote"))
                        {
                            dirEntry[i].setFileSize(Integer.parseInt(fSize[i]));

                            if(dirEntry[i].file.endsWith("/"))
                            {
                                dirEntry[i].setDirectory();
                            }
                            else
                            {
                                dirEntry[i].setFile();
                            }
                            if(dirEntry[i].file.endsWith("@"))
                            {
                                dirEntry[i].setLink();
                            }
                        }

                    }
                }
                else
                {
                    Log.debug("Not a directory: " + path);
                }
            }
            //System.out.println("length: "+dirEntry.length);
        }
        jl.setModel(jlm);
    }

    // 20011213:rb - Added logic for MSDOS style root dir
    public boolean setPath(String p)
    {
        if(!p.startsWith("/") && !p.startsWith(":",1))
        {
            p = path + p;
        }
        File f = new File(p);

        if(f.exists())
        {
            try
            {
                path = f.getCanonicalPath();
                path = path.replace('\\','/');

                if(!path.endsWith("/"))
                {
                    path = path + "/";
                }
            }
            catch(IOException ex)
            {
                Log.debug("Error: can not get pathname (local)!");
            }

            clear();
            if(JFtp.remoteDir != null)
            {
	        FtpConnection con = ConnectionHandler.getConnection();
		if(con != null)
		{
		 con.setLocalPath(path);
                 SaveSet s = new SaveSet(Settings.login_def,
                                        con.getHost(),
                                        con.getUsername(),
                                        con.getPassword(),
                                        Integer.toString(con.getPort()),
                                        con.getCachedPWD(),
                                        con.getLocalPath()
                                       );
		}
            }

            pathChanged = true;
            gui(false);
        }
        else
        {
            Log.debug("(local) No such path: \"" + p + "\"");
            return false;
        }

	//Log.debug("CWD (local) : " + p);
        return true;
    }

    public void actionPerformed(ActionEvent e)
    {
     if(JFtp.uiBlocked) return;

        if(e.getActionCommand().equals("rm"))
        {

            for(int i=0; i<length;i++)
            {
                if(dirEntry[i].selected)
                {
	        	AutoRemover armv =
				new AutoRemover(dirEntry[i].file,type);
                }
            }

	    if(type.equals("local")) JFtp.localUpdate();
            else if(type.equals("remote")) JFtp.remoteUpdate();
        }

        if(e.getActionCommand().equals("mkdir"))
        {
            Creator c = new Creator("Create:",type);
        }

        if(e.getActionCommand().equals("cmd"))
        {
            RemoteCommand rc = new RemoteCommand();
        }

        if(e.getActionCommand().equals("cd"))
        {
            if(type.equals("remote"))
            {
                PathChanger pthc = new PathChanger("remote");
            }
            if(type.equals("local"))
            {
 		JFileChooser chooser = new JFileChooser(path);
		chooser.setDialogTitle("Choose directory");
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

    		int returnVal = chooser.showOpenDialog(this);
    		if(returnVal == JFileChooser.APPROVE_OPTION) {
		 File f = chooser.getSelectedFile();
		 String tmp = f.getPath();

		 setPath(tmp);
    		}
            }
        }

        if(e.getActionCommand().equals("fresh"))
        {
            fresh();
        }

        if(e.getActionCommand().equals("->"))
        {
	 blockedTransfer(-1);
        }

        if(e.getActionCommand().equals("<-"))
        {
	 blockedTransfer(-1);
        }
    }


    public synchronized void blockedTransfer(int index)
    {
     tmpindex = index;

	 Runnable r = new Runnable()
	 {
	  public void run()
	  {        // --------------- local -------------------
	    if(type.equals("local")) 
	    {
	    	lock();
            	transfer(tmpindex);
            	JFtp.remoteDir.fresh();
	    	unlock();
	    }
	    else // --------------- remote -------------------
	    {
	    	if(!Settings.getEnableMultiThreading()) lock();
	    
		transfer(tmpindex);
	    
            	if(!Settings.getEnableMultiThreading()) 
	    	{
	    		JFtp.localDir.fresh();
	    		unlock();
	    	} // --------------- remote -------------------
	    }
	  }
	 };

	 Thread t = new Thread(r);
	 t.start();
    }

    public void lock()
    {
    	JFtp.uiBlocked = true;
	JFtp.localDir.jl.setEnabled(false);
	JFtp.remoteDir.jl.setEnabled(false);
    }

    public void unlock()
    {
    	JFtp.uiBlocked = false;
	JFtp.localDir.jl.setEnabled(true);
	JFtp.remoteDir.jl.setEnabled(true);
    }

    public void fresh()
    {
        if(type.equals("local"))
        {
            setPath(path);
        }
        else if(type.equals("remote"))
        {
            ConnectionHandler.getConnection().chdir(path);
        }
    }

    public void clear()
    {
        dirEntry = null;
        length = 1;
        files = null;
        pos = 0;
    }

    /** this manages the selections */
    public void valueChanged(ListSelectionEvent e)
    {
        if (e.getValueIsAdjusting() == false)
        {
	    							  //  ui refresh bugfix
	    int index = jl.getSelectedIndex()-1;
            if (jl.getSelectedIndex() == -1 || dirEntry == null || dirEntry.length < index || dirEntry[index] == null)
            {
                //No selection, disable fire button.
                //System.out.println("List selection: no change");
            }
            else
            {        // -------------------- local --------------------------
                if (type.equals("local"))
                {
                    String tgt=(String)jl.getSelectedValue().toString();

                    if(index >= 0)
                    {
                        for (int i=0; i<dirEntry.length; i++)
                        {
                            dirEntry[i].setSelected(jl.isSelectedIndex(i+1));
                        }
                    }
                }
                else // -------------------- remote --------------------------
                {
                    String tgt=(String)jl.getSelectedValue().toString();

		    //if (index < 0)
                    //{
                        //ConnectionHandler.getConnection().chdir(path+tgt);
                    //}
                    if(index >= 0)
                    {
                        for (int i=0; i<dirEntry.length; i++)
                        {
                            dirEntry[i].setSelected(jl.isSelectedIndex(i+1));
                        }
                    }
                }     // -------------------- remote --------------------------
            }
        }
    }

    public void updateProgress(DataConnection con, String file, String type, int bytes)
    {
    	if(dList == null || dirEntry == null) return;

    	file = file.substring(file.lastIndexOf("/")+1);
	int s = 0;

	for(int i=0; i<dirEntry.length; i++)
	{
		if(dirEntry[i] == null) continue;
		if(dirEntry[i].toString().equals(file))
		{
		 s = dirEntry[i].getRawSize();
		 break;
		} 
		
	}
	
    	dList.updateList(con,file,type,bytes,s);
    }
    
    public void connectionInitialized(FtpConnection con)
    {
    	ConnectionHandler.getConnection().setLocalPath(JFtp.localDir.path);
    	con.setLocalPath(JFtp.localDir.path);
    }

    public void connectionFailed(FtpConnection con)
    {
    	this.con = con;
	HFrame h = new HFrame();
	h.getContentPane().setLayout(new BorderLayout(10,10));
	h.setTitle("Connection failed!");
	h.setLocation(150,200);
	JTextArea text = new JTextArea();
	h.getContentPane().add("Center",text);
	text.setText(" ---------------- Output -----------------\n" + JFtp.log.getText());
	JFtp.log.setText("");
	text.setEditable(false);
	h.pack();
	h.show();
    }

    public void updateRemoteDirectory(FtpConnection con)
    {
	path = con.getPWD();
        clear();

	SaveSet s = new SaveSet(Settings.login_def,
                                        con.getHost(),
                                        con.getUsername(),
                                        con.getPassword(),
                                        Integer.toString(con.getPort()),
                                        path,
                                        con.getLocalPath()
                                       );
        pathChanged = true;
        gui(false);
	JFtp.localUpdate();
    }    
    
    private String cutPath(String s)
    {
        int maxlabel = 64;
        if(s.length() > maxlabel)
        {
            while(true)
            {
                s = StringUtils.cutAfter(s,'/');
                if(s.length() < 16)
                {
                    StringBuffer sb = new StringBuffer(s);
                    sb.insert(0,".../");
                    return sb.toString();
                }
            }
        }
        return s;
    }

}
