/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.sf.jftp.gui;

import net.sf.jftp.gui.framework.*;
import net.sf.jftp.config.*;
import net.sf.jftp.util.*;
import net.sf.jftp.net.*;
import net.sf.jftp.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class DownloadList extends HPanel implements ActionListener
{
 
private JTextArea text = new JTextArea();
private Hashtable downloads = new Hashtable();
private Hashtable connections = new Hashtable();
private boolean working = false;

private HImageButton cancel = new HImageButton(Settings.deleteImage,"delete","Cancel selected transfer...",this);
private HImageButton clear = new HImageButton(Settings.clearImage,"clear","Remove old/stalled items from output...",this);
private HImageButton rotate = new HImageButton(Settings.cmdImage,"rotate","Toggle selected transfer...",this);

private int rotator = 0;

private static final String SEP = "--> ";

public DownloadList()
{
	setLayout(new BorderLayout());
	HPanel cmdP = new HPanel();
	cmdP.add(clear);
	cmdP.add(cancel);
	cmdP.add(rotate);
        clear.setSize(24,24);
	clear.setBorder(true);	
        cancel.setSize(24,24);
	cancel.setBorder(true);	
        rotate.setSize(24,24);
	rotate.setBorder(true);	
	
	JScrollPane dP = new JScrollPane(text);
	add("South",cmdP);
	add("Center",dP);
}

public void actionPerformed(ActionEvent e)
{
	if(e.getActionCommand().equals("delete"))
	{
	
		String cmd = text.getText();
		if(!(cmd.indexOf(SEP) < 0)) cmd = cmd.substring(cmd.indexOf(SEP));
		
		if(cmd.indexOf(Download.QUEUED) >= 0)
		{
			 cmd = getFile(cmd.substring(0, cmd.indexOf("\n")));
		
			 try 
			 {
	    			Download d = (Download) ConnectionHandler.getConnections().get(cmd);
				d.work = false;
			 }
			 catch(Exception ex)
			 {
				ex.printStackTrace();
		 	}
		}
		else
		{ 
			cmd = getFile(cmd.substring(0, cmd.indexOf("\n")));
			
			DataConnection con = (DataConnection) connections.get(cmd);
			try 
			{
				con.sock.close();
				//con.getCon().abort();
				//if(Settings.getEnableMultiThreading()) con.getCon().disconnect();
			}
			catch(Exception ex) 
			{
				ex.printStackTrace();
			}
		 }
	}
	else if(e.getActionCommand().equals("clear"))
	{
		//FtpConnection con = ConnectionHandler.getConnection();
		//ConnectionHandler.fireProgressClear(con);
		downloads = new Hashtable();
		connections = new Hashtable();
		updateArea();
	}
	else if(e.getActionCommand().equals("rotate"))
	{
		rotator++;
		if(rotator >= downloads.size()) rotator = 0;
	}
}

public void updateList(DataConnection con, String file, String type, int bytes, int size)
{
	String message =  type + ": <" + file +"> ";
		
	String tmp; 
	int s = (int)(size/1024);
	if(s > 0) tmp = Integer.toString(s);
	else tmp = "?";
	
	if(type.equals(DataConnection.GET) || type.equals(DataConnection.PUT)) message = message + (int)(bytes/1024) + " / " + tmp + " kb";
	downloads.put(file, message);
	if(con != null) connections.put(file, con);
	
	if(type.equals(DataConnection.FINISHED) || type.equals(DataConnection.FAILED))
	{
		//try { Thread.sleep(2000); }
		//catch(Exception ex){}
		JFtp.localUpdate();
		//JFtp.remoteUpdate();
	}
	
	updateArea();
}

/*
public void clear()
{
	Hashtable newTable = new Hashtable();
	Enumeration e = downloads.elements();
	Enumeration k = downloads.keys();
	
	while(e.hasMoreElements())
	{
		String msg = (String) e.nextElement();
		String file = (String) k.nextElement();
		
		if(!msg.startsWith(DataConnection.FINISHED) && !msg.startsWith(DataConnection.FAILED))
		{	
			newTable.put(file,msg);
		}
		
		downloads = newTable;
		updateArea();
	}
}
*/

private synchronized String[] rotate()
{
	
	int r = downloads.size();
	String[] f = new String[r];
	int i = 0;
	//int back = rotator;
	
	Enumeration k = downloads.elements();
	
	while(k.hasMoreElements())
	{
		/*
		if(i==0)
		{
			
			while(rotator < r && k.hasMoreElements())
			{
				f[rotator] = (String) k.nextElement();
				rotator++;
			}
		}
		*/
		
		//if(!k.hasMoreElements()) break;
		
		if(i == rotator) f[i] = SEP + (String) k.nextElement();
		else f[i] = (String) k.nextElement();
		
		i++;
	}
	
	//rotator = back;
	 
	return f;
}

private synchronized void updateArea()
{
	text.setText("");

	/*
	 Enumeration e = downloads.elements();
	 
 	 while(e.hasMoreElements())
	 {
		String tmp = (String) e.nextElement();
		text.append(tmp +"\n");
		//System.out.println(tmp);
	 }

	*/

	String[] f = rotate();
	
	 for(int i=0; i<f.length; i++)
	 {
		text.append(f[i] +"\n");
	 }
}


private String getFile(String msg)
{
	String f = msg.substring(msg.indexOf("<")+1);
	f = f.substring(0,f.lastIndexOf(">"));
	//System.out.println(f);
	return f;			
}


}
