/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.sf.jftp.gui;

import net.sf.jftp.gui.framework.*;
import net.sf.jftp.config.Settings;
import net.sf.jftp.net.*;
import net.sf.jftp.util.*;
import net.sf.jftp.JFtp;

import java.awt.*;
import java.awt.event.*;

public class StatusPanel extends HPanel implements ActionListener
{
    private HImageButton newcon = new HImageButton(Settings.hostImage,"newcon","New Connection...",this);
    private HImageButton info = new HImageButton(Settings.infoImage,"info","About JFtp...",this);

    public HImageButton close = new HImageButton(Settings.closeImage,"close","Exit JFtp...",this);
    private HImageButton list = new HImageButton(Settings.listImage,"list","Show remote listing...",this);
    private HImageButton type = new HImageButton(Settings.typeImage,"type","Toggle transfer type...",this);

    private StatusCanvas status = new StatusCanvas();
    private JFtp jftp;
    
    public StatusPanel(JFtp jftp)
    {
		this.jftp = jftp;
		setLayout(new FlowLayout(FlowLayout.LEFT));
		//setLayout(new GridLayout(1,5));

	newcon.setBorder(true);	
	close.setBorder(true);	
	info.setBorder(true);	
	list.setBorder(true);	
	type.setBorder(true);	
		
        add(newcon);
        newcon.setSize(24,24);

        add(close);
        close.setSize(24,24);

        add(list);
        list.setSize(24,24);
	
        add(type);
        type.setSize(24,24);
	
	add(info);
        info.setSize(24,24);
	
        add(status);

        validate();
        setFont(GUIDefaults.menuFont);
        setVisible(true);
    }

    public void status(String msg)
    {
        status.setText(msg);
    }

    public String getHost()
    {
        return status.getHost();
    }

    public void setHost(String host)
    {
        status.setHost(host);
    }

    public void actionPerformed(ActionEvent e)
    {
        if(e.getActionCommand().equals("info"))
        {
            java.net.URL url = ClassLoader.getSystemResource(Settings.readme);
            if(url == null)
            {
                url = HImage.class.getResource("/"+Settings.readme);
            }
            Displayer d = new Displayer(url);
        }
        else if(e.getActionCommand().equals("close"))
        {
	 try {
            ConnectionHandler.getConnection().disconnect();
	 } catch(Exception ex) {
	 }
            System.exit(0);
        }
        else if(e.getActionCommand().equals("newcon"))
        {
            ConnectionHandler.getConnection().disconnect();

            // Switch windows
	    jftp.mainFrame.setVisible(false);
            HostChooser hc = new HostChooser();
            hc.toFront();
		    hc.setModal(true);
            hc.update();
        }
        else if(e.getActionCommand().equals("list"))
        {
            try
            {
                java.net.URL url = new java.io.File(Settings.ls_out).toURL();
                Displayer d = new Displayer(url);
            }
            catch(java.net.MalformedURLException ex)
            {
                ex.printStackTrace();
            }
        }
	else if(e.getActionCommand().equals("type"))
	{
	 FtpConnection con = ConnectionHandler.getConnection();
	 String type = con.getTypeNow();
	 boolean ret = false;
	 
	 if(type.equals(FtpConnection.ASCII))
	 {
	 	ret = con.type(FtpConnection.BINARY);
	 } 
	 else if(type.equals(FtpConnection.BINARY))
	 {
	 	ret = con.type(FtpConnection.EBCDIC);
	 }
	  
	 if(type.equals(FtpConnection.EBCDIC) || (!ret && !type.equals(FtpConnection.L8)))
	 {
	 	ret = con.type(FtpConnection.L8);
	 }

	 if(!ret) 
	 {
	 	con.type(FtpConnection.ASCII);
		Log.debug("Warning: type should be \"I\" if you want to transfer binary files!");
	 }
	 
	 status.setText("Type is now " + con.getTypeNow());	
	}
    }
    

    // <newimg>public void paintComponent(Graphics g)
    // <newimg>{
    // <newimg>	int x = getSize().width;
    // <newimg>while(x > -100) {
    // <newimg>		g.drawImage(HImage.getImage(this, Settings.statusBackImage), x, 0, this);
    // <newimg>	x -= 100;
    // <newimg>}
    // <newimg>}
  

    public Insets getInsets()
    {
        Insets in = super.getInsets();
        return new Insets(in.top,in.left,in.bottom,in.right);
    }
}
