/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package net.sf.jftp.net;

import net.sf.jftp.util.*;
import java.util.*;

/** may be extended to support multiple connections in future */
public class ConnectionHandler
{
    private static FtpConnection connection = null;
    private static ConnectionListener listener = null;

    private static Hashtable connections = new Hashtable();
    
    /** sets the current connection */
    public static void setConnection(FtpConnection con)
    {
	//Log.debug("Set Connection...");
        connection = con;
    }

    /** returns the current connection */
    public static FtpConnection getConnection()
    {
    /*
	if(connection == null)
	{
		Log.debug("No Connection!");
	}
    */
        return connection;
    }

    /** we should support multiple listeners in future */
    public static void setConnectionListener(ConnectionListener l)
    {
    	listener = l;
    }

    public static void addConnection(String file, Download dl)
    {
    	connections.put(file, dl);
    }
    
    public static void removeConnection(String file)
    {
    	connections.remove(file);
    }
    
    public static Hashtable getConnections()
    {
    	return connections;
    }
    
    public static int getConnectionSize()
    {
    	int size = 0;
	Enumeration e = connections.elements();
	
	while(e.hasMoreElements())
	{
		Download d = (Download) e.nextElement();
		if(d.hasStarted()) size++;
	}
	
	
   	return size;
    }
        
    /** remote directory has changed */
    public static void fireDirectoryUpdate(FtpConnection con)
    {
    	if(listener == null) return;
	else listener.updateRemoteDirectory(con);
    }
    
    /** progress update */
    public static void fireProgressUpdate(DataConnection con, String file, String type, int bytes)
    {
    	if(listener == null) return;
	else listener.updateProgress(con, file, type, bytes);
    }
    
    /*
    public static void fireProgressClear(FtpConnection con)
    {
    	if(listener == null) return;
	else listener.clearProgress(con);
    }
    *

    /** connection is there and user logged in */
    public static void fireConnectionInitialized(FtpConnection con)
    {
    	if(listener == null) return;
	else listener.connectionInitialized(con);
    }

    /** we are not logged in for some reason */
    public static void fireConnectionFailed(FtpConnection con)
    {
    	if(listener == null) return;
	else listener.connectionFailed(con);
    }

}
