package net.sf.jftp.net;

import net.sf.jftp.util.*;
import net.sf.jftp.config.*;
import java.io.*;

public class Download implements Runnable
{      	

 private String host;
 private int port;
 private String localPath;
 private String remotePath;
 private String file;
 private String user;
 private String pass;
 private FtpConnection con = null;
 public boolean work = true;
 
 public Thread runner;
 
 private int stat = 0;
 private boolean started = false;
 
 public static final String QUEUED = "QUEUED";
 public static final String REMOVED = "REMOVED";
 

 public Download(String host, int port, String localPath, String remotePath, String file, String user, String pass)
 {
 	this.host = host;
	this.port = port;
	this.localPath = localPath;
	this.remotePath = remotePath;
	this.file = file;
	this.user = user;
	this.pass = pass;
	
	runner = new Thread(this);
	runner.setPriority(Thread.MIN_PRIORITY);
	runner.start();
 }
 
 public void run()
 {
 	ConnectionHandler.addConnection(file, this);
	
 	while(ConnectionHandler.getConnectionSize() >= Settings.getMaxConnections() && ConnectionHandler.getConnectionSize() > 0)
	{
	
		if(!work)
		{
			ConnectionHandler.fireProgressUpdate(null, file, REMOVED, -1);
			break;
		}
		
		try 
		{
			runner.sleep(400);
  			ConnectionHandler.fireProgressUpdate(null, file, QUEUED, -1);
		}
		catch(Exception ex) {}
	}
	
	if(!work) 
	{
		ConnectionHandler.removeConnection(file);
		return;
	}
	
	started = true;
	
 	//Log.debug("Download started: " + file);
	try {runner.sleep(2000);}
		catch(Exception ex) {}
	
	con = new FtpConnection(host, port, remotePath);
	File f = new File(localPath);
        con.setLocalPath(f.getAbsolutePath());
	String status = con.login(user, pass);
	
	if(status.equals(FtpConnection.LOGIN_OK))
	{
		stat = 1;
		con.download(file);
		//Log.debug("Download finished: " + file);
	}
	else
	{
		stat = -1;
		//Log.debug("Download failed: " + file);
	}
	
	ConnectionHandler.removeConnection(file);
 }
 
 public int getStatus()
 {
 	return stat;
 }
 
 public boolean hasStarted()
 {
 	return started;
 }
	
}
